@@ -18,16 +18,17 @@ use zellij_client::{
 use zellij_server::{os_input_output::get_server_os_input, start_server as start_server_impl};
 use zellij_utils::{
     cli::{CliArgs, Command, SessionCommand, Sessions},
-    data::ConnectToSession,
+    data::{ConnectToSession, LayoutInfo},
     envs,
     input::{
         actions::Action,
         config::{Config, ConfigError},
+        layout::Layout,
         options::Options,
     },
     miette::{Report, Result},
     nix,
-    setup::Setup,
+    setup::{find_default_config_dir, get_layout_dir, Setup},
 };
 
 pub(crate) use crate::sessions::list_sessions;
@@ -206,14 +207,14 @@ pub(crate) fn send_action_to_session(
                         "Session '{}' not found. The following sessions are active:",
                         session_name
                     );
-                    list_sessions(false, false);
+                    list_sessions(false, false, true);
                     std::process::exit(1);
                 }
             } else if let Ok(session_name) = envs::get_session_name() {
                 attach_with_cli_client(cli_action, &session_name, config);
             } else {
                 eprintln!("Please specify the session name to send actions to. The following sessions are active:");
-                list_sessions(false, false);
+                list_sessions(false, false, true);
                 std::process::exit(1);
             }
         },
@@ -356,6 +357,7 @@ fn attach_with_session_name(
                         .collect(),
                     false,
                     false,
+                    true,
                 );
                 process::exit(1);
             },
@@ -373,7 +375,7 @@ fn attach_with_session_name(
             ActiveSession::One(session_name) => ClientInfo::Attach(session_name, config_options),
             ActiveSession::Many => {
                 println!("Please specify the session to attach to, either by using the full name or a unique prefix.\nThe following sessions are active:");
-                list_sessions(false, false);
+                list_sessions(false, false, true);
                 process::exit(1);
             },
         },
@@ -383,27 +385,29 @@ fn attach_with_session_name(
 pub(crate) fn start_client(opts: CliArgs) {
     // look for old YAML config/layout/theme files and convert them to KDL
     convert_old_yaml_files(&opts);
-    let (config, layout, config_options) = match Setup::from_cli_args(&opts) {
-        Ok(results) => results,
-        Err(e) => {
-            if let ConfigError::KdlError(error) = e {
-                let report: Report = error.into();
-                eprintln!("{:?}", report);
-            } else {
-                eprintln!("{}", e);
-            }
-            process::exit(1);
-        },
-    };
+    let (config, layout, config_options, config_without_layout, config_options_without_layout) =
+        match Setup::from_cli_args(&opts) {
+            Ok(results) => results,
+            Err(e) => {
+                if let ConfigError::KdlError(error) = e {
+                    let report: Report = error.into();
+                    eprintln!("{:?}", report);
+                } else {
+                    eprintln!("{}", e);
+                }
+                process::exit(1);
+            },
+        };
     let mut reconnect_to_session: Option<ConnectToSession> = None;
     let os_input = get_os_input(get_client_os_input);
     loop {
         let os_input = os_input.clone();
-        let config = config.clone();
-        let layout = layout.clone();
+        let mut config = config.clone();
+        let mut layout = layout.clone();
         let mut config_options = config_options.clone();
         let mut opts = opts.clone();
         let mut is_a_reconnect = false;
+        let mut should_create_detached = false;
 
         if let Some(reconnect_to_session) = &reconnect_to_session {
             // this is integration code to make session reconnects work with this existing,
@@ -414,6 +418,7 @@ pub(crate) fn start_client(opts: CliArgs) {
                 opts.command = Some(Command::Sessions(Sessions::Attach {
                     session_name: reconnect_to_session.name.clone(),
                     create: true,
+                    create_background: false,
                     force_run_commands: false,
                     index: None,
                     options: None,
@@ -423,6 +428,46 @@ pub(crate) fn start_client(opts: CliArgs) {
                 opts.session = None;
                 config_options.attach_to_session = None;
             }
+
+            if let Some(reconnect_layout) = &reconnect_to_session.layout {
+                let layout_dir = config.options.layout_dir.clone().or_else(|| {
+                    get_layout_dir(opts.config_dir.clone().or_else(find_default_config_dir))
+                });
+                let new_session_layout = match reconnect_layout {
+                    LayoutInfo::BuiltIn(layout_name) => Layout::from_default_assets(
+                        &PathBuf::from(layout_name),
+                        layout_dir.clone(),
+                        config_without_layout.clone(),
+                    ),
+                    LayoutInfo::File(layout_name) => Layout::from_path_or_default(
+                        Some(&PathBuf::from(layout_name)),
+                        layout_dir.clone(),
+                        config_without_layout.clone(),
+                    ),
+                };
+                match new_session_layout {
+                    Ok(new_session_layout) => {
+                        // here we make sure to override both the layout and the config, but we do
+                        // this with an instance of the config before it was merged with the
+                        // layout configuration of the previous iteration of the loop, since we do
+                        // not want it to mix with the config of this session
+                        let (new_layout, new_layout_config) = new_session_layout;
+                        layout = new_layout;
+                        if let Some(cwd) = reconnect_to_session.cwd.as_ref() {
+                            layout.add_cwd_to_layout(cwd);
+                        }
+                        let mut new_config = config_without_layout.clone();
+                        let _ = new_config.merge(new_layout_config.clone());
+                        config = new_config;
+                        config_options =
+                            config_options_without_layout.merge(new_layout_config.options);
+                    },
+                    Err(e) => {
+                        log::error!("Failed to parse new session layout: {:?}", e);
+                    },
+                }
+            }
+
             is_a_reconnect = true;
         }
 
@@ -433,6 +478,7 @@ pub(crate) fn start_client(opts: CliArgs) {
         if let Some(Command::Sessions(Sessions::Attach {
             session_name,
             create,
+            create_background,
             force_run_commands,
             index,
             options,
@@ -444,9 +490,14 @@ pub(crate) fn start_client(opts: CliArgs) {
                 },
                 None => config_options,
             };
+            should_create_detached = create_background;
 
             let client = if let Some(idx) = index {
-                attach_with_session_index(config_options.clone(), idx, create)
+                attach_with_session_index(
+                    config_options.clone(),
+                    idx,
+                    create || should_create_detached,
+                )
             } else {
                 let session_exists = session_name
                     .as_ref()
@@ -454,7 +505,10 @@ pub(crate) fn start_client(opts: CliArgs) {
                     .unwrap_or(false);
                 let resurrection_layout =
                     session_name.as_ref().and_then(|s| resurrection_layout(&s));
-                if create && !session_exists && resurrection_layout.is_none() {
+                if (create || should_create_detached)
+                    && !session_exists
+                    && resurrection_layout.is_none()
+                {
                     session_name.clone().map(start_client_plan);
                 }
                 match (session_name.as_ref(), resurrection_layout) {
@@ -464,7 +518,11 @@ pub(crate) fn start_client(opts: CliArgs) {
                         }
                         ClientInfo::Resurrect(session_name.clone(), resurrection_layout)
                     },
-                    _ => attach_with_session_name(session_name, config_options.clone(), create),
+                    _ => attach_with_session_name(
+                        session_name,
+                        config_options.clone(),
+                        create || should_create_detached,
+                    ),
                 }
             };
 
@@ -498,6 +556,7 @@ pub(crate) fn start_client(opts: CliArgs) {
                 tab_position_to_focus,
                 pane_id_to_focus,
                 is_a_reconnect,
+                should_create_detached,
             );
         } else {
             if let Some(session_name) = opts.session.clone() {
@@ -512,6 +571,7 @@ pub(crate) fn start_client(opts: CliArgs) {
                     None,
                     None,
                     is_a_reconnect,
+                    should_create_detached,
                 );
             } else {
                 if let Some(session_name) = config_options.session_name.as_ref() {
@@ -552,6 +612,7 @@ pub(crate) fn start_client(opts: CliArgs) {
                                 None,
                                 None,
                                 is_a_reconnect,
+                                should_create_detached,
                             );
                         },
                         _ => {
@@ -566,6 +627,7 @@ pub(crate) fn start_client(opts: CliArgs) {
                                 None,
                                 None,
                                 is_a_reconnect,
+                                should_create_detached,
                             );
                         },
                     }
@@ -589,6 +651,7 @@ pub(crate) fn start_client(opts: CliArgs) {
                     None,
                     None,
                     is_a_reconnect,
+                    should_create_detached,
                 );
             }
         }
@@ -625,3 +688,23 @@ fn generate_unique_session_name() -> String {
         process::exit(1);
     }
 }
+
+pub(crate) fn list_aliases(opts: CliArgs) {
+    let (config, _layout, _config_options, _config_without_layout, _config_options_without_layout) =
+        match Setup::from_cli_args(&opts) {
+            Ok(results) => results,
+            Err(e) => {
+                if let ConfigError::KdlError(error) = e {
+                    let report: Report = error.into();
+                    eprintln!("{:?}", report);
+                } else {
+                    eprintln!("{}", e);
+                }
+                process::exit(1);
+            },
+        };
+    for alias in config.plugins.list() {
+        println!("{}", alias);
+    }
+    process::exit(0);
+}
