@@ -15,7 +15,7 @@ use zellij_utils::{
     data::EventType,
     data::PluginCapabilities,
     input::command::TerminalAction,
-    input::layout::{Layout, RunPlugin, RunPluginLocation},
+    input::layout::{Layout, PluginUserConfiguration, RunPlugin, RunPluginLocation},
     input::plugins::PluginConfig,
     ipc::ClientAttributes,
 };
@@ -157,13 +157,19 @@ impl PluginMap {
     pub fn all_plugin_ids_for_plugin_location(
         &self,
         plugin_location: &RunPluginLocation,
+        plugin_configuration: &PluginUserConfiguration,
     ) -> Result<Vec<PluginId>> {
         let err_context = || format!("Failed to get plugin ids for location {plugin_location}");
         let plugin_ids: Vec<PluginId> = self
             .plugin_assets
             .iter()
             .filter(|(_, (running_plugin, _subscriptions, _workers))| {
-                &running_plugin.lock().unwrap().plugin_env.plugin.location == plugin_location
+                let running_plugin = running_plugin.lock().unwrap();
+                let running_plugin_location = &running_plugin.plugin_env.plugin.location;
+                let running_plugin_configuration =
+                    &running_plugin.plugin_env.plugin.userspace_configuration;
+                running_plugin_location == plugin_location
+                    && running_plugin_configuration == plugin_configuration
             })
             .map(|((plugin_id, _client_id), _)| *plugin_id)
             .collect();
@@ -172,6 +178,49 @@ impl PluginMap {
         }
         Ok(plugin_ids)
     }
+    pub fn clone_plugin_assets(
+        &self,
+    ) -> HashMap<RunPluginLocation, HashMap<PluginUserConfiguration, Vec<(PluginId, ClientId)>>>
+    {
+        let mut cloned_plugin_assets: HashMap<
+            RunPluginLocation,
+            HashMap<PluginUserConfiguration, Vec<(PluginId, ClientId)>>,
+        > = HashMap::new();
+        for ((plugin_id, client_id), (running_plugin, _, _)) in self.plugin_assets.iter() {
+            let running_plugin = running_plugin.lock().unwrap();
+            let running_plugin_location = &running_plugin.plugin_env.plugin.location;
+            let running_plugin_configuration =
+                &running_plugin.plugin_env.plugin.userspace_configuration;
+            match cloned_plugin_assets.get_mut(running_plugin_location) {
+                Some(location_map) => match location_map.get_mut(running_plugin_configuration) {
+                    Some(plugin_instances_info) => {
+                        plugin_instances_info.push((*plugin_id, *client_id));
+                    },
+                    None => {
+                        location_map.insert(
+                            running_plugin_configuration.clone(),
+                            vec![(*plugin_id, *client_id)],
+                        );
+                    },
+                },
+                None => {
+                    let mut location_map = HashMap::new();
+                    location_map.insert(
+                        running_plugin_configuration.clone(),
+                        vec![(*plugin_id, *client_id)],
+                    );
+                    cloned_plugin_assets.insert(running_plugin_location.clone(), location_map);
+                },
+            }
+        }
+        cloned_plugin_assets
+    }
+    pub fn all_plugin_ids(&self) -> Vec<(PluginId, ClientId)> {
+        self.plugin_assets
+            .iter()
+            .map(|((plugin_id, client_id), _)| (*plugin_id, *client_id))
+            .collect()
+    }
     pub fn insert(
         &mut self,
         plugin_id: PluginId,
@@ -197,10 +246,12 @@ impl PluginMap {
                         .plugin
                         .userspace_configuration
                         .clone();
+                    let initial_cwd = running_plugin.plugin_env.plugin.initial_cwd.clone();
                     Some(RunPlugin {
                         _allow_exec_host_cmd: false,
                         location: run_plugin_location,
                         configuration: run_plugin_configuration,
+                        initial_cwd,
                     })
                 } else {
                     None
@@ -218,7 +269,7 @@ pub struct PluginEnv {
     pub permissions: Arc<Mutex<Option<HashSet<PermissionType>>>>,
     pub senders: ThreadSenders,
     pub wasi_env: WasiEnv,
-    pub tab_index: usize,
+    pub tab_index: Option<usize>,
     pub client_id: ClientId,
     #[allow(dead_code)]
     pub plugin_own_data_dir: PathBuf,
@@ -228,6 +279,8 @@ pub struct PluginEnv {
     pub default_shell: Option<TerminalAction>,
     pub default_layout: Box<Layout>,
     pub plugin_cwd: PathBuf,
+    pub input_pipes_to_unblock: Arc<Mutex<HashSet<String>>>,
+    pub input_pipes_to_block: Arc<Mutex<HashSet<String>>>,
 }
 
 impl PluginEnv {
