@@ -18,11 +18,14 @@ use std::{
 use wasmer::{imports, AsStoreMut, Function, FunctionEnv, FunctionEnvMut, Imports};
 use wasmer_wasi::WasiEnv;
 use zellij_utils::data::{
-    CommandType, ConnectToSession, HttpVerb, PermissionStatus, PermissionType, PluginPermission,
+    CommandType, ConnectToSession, FloatingPaneCoordinates, HttpVerb, LayoutInfo, MessageToPlugin,
+    PermissionStatus, PermissionType, PluginPermission,
 };
 use zellij_utils::input::permission::PermissionCache;
-
-use url::Url;
+use zellij_utils::{
+    interprocess::local_socket::LocalSocketStream,
+    ipc::{ClientToServerMsg, IpcSenderWithContext},
+};
 
 use crate::{panes::PaneId, screen::ScreenInstruction};
 
@@ -36,7 +39,7 @@ use zellij_utils::{
     input::{
         actions::Action,
         command::{RunCommand, RunCommandAction, TerminalAction},
-        layout::{Layout, PluginUserConfiguration, RunPlugin, RunPluginLocation},
+        layout::{Layout, RunPluginOrAlias},
         plugins::PluginType,
     },
     plugin_api::{
@@ -58,6 +61,7 @@ macro_rules! apply_action {
             $env.plugin_env.client_attributes.clone(),
             $env.plugin_env.default_shell.clone(),
             $env.plugin_env.default_layout.clone(),
+            None,
         ) {
             log::error!("{}: {:?}", $error_message(), e);
         }
@@ -111,19 +115,20 @@ fn host_run_plugin_command(env: FunctionEnvMut<ForeignFunctionEnv>) {
                     PluginCommand::GetPluginIds => get_plugin_ids(env),
                     PluginCommand::GetZellijVersion => get_zellij_version(env),
                     PluginCommand::OpenFile(file_to_open) => open_file(env, file_to_open),
-                    PluginCommand::OpenFileFloating(file_to_open) => {
-                        open_file_floating(env, file_to_open)
+                    PluginCommand::OpenFileFloating(file_to_open, floating_pane_coordinates) => {
+                        open_file_floating(env, file_to_open, floating_pane_coordinates)
                     },
                     PluginCommand::OpenTerminal(cwd) => open_terminal(env, cwd.path.try_into()?),
-                    PluginCommand::OpenTerminalFloating(cwd) => {
-                        open_terminal_floating(env, cwd.path.try_into()?)
+                    PluginCommand::OpenTerminalFloating(cwd, floating_pane_coordinates) => {
+                        open_terminal_floating(env, cwd.path.try_into()?, floating_pane_coordinates)
                     },
                     PluginCommand::OpenCommandPane(command_to_run) => {
                         open_command_pane(env, command_to_run)
                     },
-                    PluginCommand::OpenCommandPaneFloating(command_to_run) => {
-                        open_command_pane_floating(env, command_to_run)
-                    },
+                    PluginCommand::OpenCommandPaneFloating(
+                        command_to_run,
+                        floating_pane_coordinates,
+                    ) => open_command_pane_floating(env, command_to_run, floating_pane_coordinates),
                     PluginCommand::SwitchTabTo(tab_index) => switch_tab_to(env, tab_index),
                     PluginCommand::SetTimeout(seconds) => set_timeout(env, seconds),
                     PluginCommand::ExecCmd(command_line) => exec_cmd(env, command_line),
@@ -223,6 +228,8 @@ fn host_run_plugin_command(env: FunctionEnvMut<ForeignFunctionEnv>) {
                         connect_to_session.name,
                         connect_to_session.tab_position,
                         connect_to_session.pane_id,
+                        connect_to_session.layout,
+                        connect_to_session.cwd,
                     )?,
                     PluginCommand::DeleteDeadSession(session_name) => {
                         delete_dead_session(session_name)?
@@ -240,6 +247,24 @@ fn host_run_plugin_command(env: FunctionEnvMut<ForeignFunctionEnv>) {
                     PluginCommand::RenameSession(new_session_name) => {
                         rename_session(env, new_session_name)
                     },
+                    PluginCommand::UnblockCliPipeInput(pipe_name) => {
+                        unblock_cli_pipe_input(env, pipe_name)
+                    },
+                    PluginCommand::BlockCliPipeInput(pipe_name) => {
+                        block_cli_pipe_input(env, pipe_name)
+                    },
+                    PluginCommand::CliPipeOutput(pipe_name, output) => {
+                        cli_pipe_output(env, pipe_name, output)?
+                    },
+                    PluginCommand::MessageToPlugin(message) => message_to_plugin(env, message)?,
+                    PluginCommand::DisconnectOtherClients => disconnect_other_clients(env),
+                    PluginCommand::KillSessions(session_list) => kill_sessions(session_list),
+                    PluginCommand::ScanHostFolder(folder_to_scan) => {
+                        scan_host_folder(env, folder_to_scan)
+                    },
+                    PluginCommand::WatchFilesystem => watch_filesystem(env),
+                    PluginCommand::DumpSessionLayout => dump_session_layout(env),
+                    PluginCommand::CloseSelf => close_self(env),
                 },
                 (PermissionStatus::Denied, permission) => {
                     log::error!(
@@ -272,6 +297,45 @@ fn subscribe(env: &ForeignFunctionEnv, event_list: HashSet<EventType>) -> Result
         ))
 }
 
+fn unblock_cli_pipe_input(env: &ForeignFunctionEnv, pipe_name: String) {
+    env.plugin_env
+        .input_pipes_to_unblock
+        .lock()
+        .unwrap()
+        .insert(pipe_name);
+}
+
+fn block_cli_pipe_input(env: &ForeignFunctionEnv, pipe_name: String) {
+    env.plugin_env
+        .input_pipes_to_block
+        .lock()
+        .unwrap()
+        .insert(pipe_name);
+}
+
+fn cli_pipe_output(env: &ForeignFunctionEnv, pipe_name: String, output: String) -> Result<()> {
+    env.plugin_env
+        .senders
+        .send_to_server(ServerInstruction::CliPipeOutput(pipe_name, output))
+        .context("failed to send pipe output")
+}
+
+fn message_to_plugin(
+    env: &ForeignFunctionEnv,
+    mut message_to_plugin: MessageToPlugin,
+) -> Result<()> {
+    if message_to_plugin.plugin_url.as_ref().map(|s| s.as_str()) == Some("zellij:OWN_URL") {
+        message_to_plugin.plugin_url = Some(env.plugin_env.plugin.location.display());
+    }
+    env.plugin_env
+        .senders
+        .send_to_plugin(PluginInstruction::MessageFromPlugin {
+            source_plugin_id: env.plugin_env.plugin_id,
+            message: message_to_plugin,
+        })
+        .context("failed to send message to plugin")
+}
+
 fn unsubscribe(env: &ForeignFunctionEnv, event_list: HashSet<EventType>) -> Result<()> {
     env.subscriptions
         .lock()
@@ -325,6 +389,15 @@ fn request_permission(env: &ForeignFunctionEnv, permissions: Vec<PermissionType>
             ));
     }
 
+    // we do this so that messages that have arrived while the user is seeing the permission screen
+    // will be cached and reapplied once the permission is granted
+    let _ = env
+        .plugin_env
+        .senders
+        .send_to_plugin(PluginInstruction::CachePluginEvents {
+            plugin_id: env.plugin_env.plugin_id,
+        });
+
     env.plugin_env
         .senders
         .send_to_screen(ScreenInstruction::RequestPluginPermissions(
@@ -337,6 +410,7 @@ fn get_plugin_ids(env: &ForeignFunctionEnv) {
     let ids = PluginIds {
         plugin_id: env.plugin_env.plugin_id,
         zellij_pid: process::id(),
+        initial_cwd: env.plugin_env.plugin_cwd.clone(),
     };
     ProtobufPluginIds::try_from(ids)
         .map_err(|e| anyhow!("Failed to serialized plugin ids: {}", e))
@@ -377,7 +451,8 @@ fn open_file(env: &ForeignFunctionEnv, file_to_open: FileToOpen) {
     let path = env.plugin_env.plugin_cwd.join(file_to_open.path);
     let cwd = file_to_open
         .cwd
-        .map(|cwd| env.plugin_env.plugin_cwd.join(cwd));
+        .map(|cwd| env.plugin_env.plugin_cwd.join(cwd))
+        .or_else(|| Some(env.plugin_env.plugin_cwd.clone()));
     let action = Action::EditFile(
         path,
         file_to_open.line_number,
@@ -385,18 +460,24 @@ fn open_file(env: &ForeignFunctionEnv, file_to_open: FileToOpen) {
         None,
         floating,
         in_place,
+        None,
     );
     apply_action!(action, error_msg, env);
 }
 
-fn open_file_floating(env: &ForeignFunctionEnv, file_to_open: FileToOpen) {
+fn open_file_floating(
+    env: &ForeignFunctionEnv,
+    file_to_open: FileToOpen,
+    floating_pane_coordinates: Option<FloatingPaneCoordinates>,
+) {
     let error_msg = || format!("failed to open file in plugin {}", env.plugin_env.name());
     let floating = true;
     let in_place = false;
     let path = env.plugin_env.plugin_cwd.join(file_to_open.path);
     let cwd = file_to_open
         .cwd
-        .map(|cwd| env.plugin_env.plugin_cwd.join(cwd));
+        .map(|cwd| env.plugin_env.plugin_cwd.join(cwd))
+        .or_else(|| Some(env.plugin_env.plugin_cwd.clone()));
     let action = Action::EditFile(
         path,
         file_to_open.line_number,
@@ -404,6 +485,7 @@ fn open_file_floating(env: &ForeignFunctionEnv, file_to_open: FileToOpen) {
         None,
         floating,
         in_place,
+        floating_pane_coordinates,
     );
     apply_action!(action, error_msg, env);
 }
@@ -415,7 +497,9 @@ fn open_file_in_place(env: &ForeignFunctionEnv, file_to_open: FileToOpen) {
     let path = env.plugin_env.plugin_cwd.join(file_to_open.path);
     let cwd = file_to_open
         .cwd
-        .map(|cwd| env.plugin_env.plugin_cwd.join(cwd));
+        .map(|cwd| env.plugin_env.plugin_cwd.join(cwd))
+        .or_else(|| Some(env.plugin_env.plugin_cwd.clone()));
+
     let action = Action::EditFile(
         path,
         file_to_open.line_number,
@@ -423,6 +507,7 @@ fn open_file_in_place(env: &ForeignFunctionEnv, file_to_open: FileToOpen) {
         None,
         floating,
         in_place,
+        None,
     );
     apply_action!(action, error_msg, env);
 }
@@ -444,7 +529,11 @@ fn open_terminal(env: &ForeignFunctionEnv, cwd: PathBuf) {
     apply_action!(action, error_msg, env);
 }
 
-fn open_terminal_floating(env: &ForeignFunctionEnv, cwd: PathBuf) {
+fn open_terminal_floating(
+    env: &ForeignFunctionEnv,
+    cwd: PathBuf,
+    floating_pane_coordinates: Option<FloatingPaneCoordinates>,
+) {
     let error_msg = || format!("failed to open file in plugin {}", env.plugin_env.name());
     let cwd = env.plugin_env.plugin_cwd.join(cwd);
     let mut default_shell = env
@@ -457,7 +546,7 @@ fn open_terminal_floating(env: &ForeignFunctionEnv, cwd: PathBuf) {
         TerminalAction::RunCommand(run_command) => Some(run_command.into()),
         _ => None,
     };
-    let action = Action::NewFloatingPane(run_command_action, None);
+    let action = Action::NewFloatingPane(run_command_action, None, floating_pane_coordinates);
     apply_action!(action, error_msg, env);
 }
 
@@ -501,7 +590,11 @@ fn open_command_pane(env: &ForeignFunctionEnv, command_to_run: CommandToRun) {
     apply_action!(action, error_msg, env);
 }
 
-fn open_command_pane_floating(env: &ForeignFunctionEnv, command_to_run: CommandToRun) {
+fn open_command_pane_floating(
+    env: &ForeignFunctionEnv,
+    command_to_run: CommandToRun,
+    floating_pane_coordinates: Option<FloatingPaneCoordinates>,
+) {
     let error_msg = || format!("failed to open command in plugin {}", env.plugin_env.name());
     let command = command_to_run.path;
     let cwd = command_to_run
@@ -520,7 +613,7 @@ fn open_command_pane_floating(env: &ForeignFunctionEnv, command_to_run: CommandT
         hold_on_close,
         hold_on_start,
     };
-    let action = Action::NewFloatingPane(Some(run_command_action), name);
+    let action = Action::NewFloatingPane(Some(run_command_action), name, floating_pane_coordinates);
     apply_action!(action, error_msg, env);
 }
 
@@ -727,6 +820,22 @@ fn show_self(env: &ForeignFunctionEnv, should_float_if_hidden: bool) {
     apply_action!(action, error_msg, env);
 }
 
+fn close_self(env: &ForeignFunctionEnv) {
+    env.plugin_env
+        .senders
+        .send_to_screen(ScreenInstruction::ClosePane(
+            PaneId::Plugin(env.plugin_env.plugin_id),
+            None,
+        ))
+        .with_context(|| format!("failed to close self"))
+        .non_fatal();
+    env.plugin_env
+        .senders
+        .send_to_plugin(PluginInstruction::Unload(env.plugin_env.plugin_id))
+        .with_context(|| format!("failed to close self"))
+        .non_fatal();
+}
+
 fn switch_to_mode(env: &ForeignFunctionEnv, input_mode: InputMode) {
     let action = Action::SwitchToMode(input_mode);
     let error_msg = || {
@@ -846,6 +955,8 @@ fn switch_session(
     session_name: Option<String>,
     tab_position: Option<usize>,
     pane_id: Option<(u32, bool)>,
+    layout: Option<LayoutInfo>,
+    cwd: Option<PathBuf>,
 ) -> Result<()> {
     // pane_id is (id, is_plugin)
     let err_context = || format!("Failed to switch session");
@@ -855,6 +966,8 @@ fn switch_session(
         name: session_name,
         tab_position,
         pane_id,
+        layout,
+        cwd,
     };
     env.plugin_env
         .senders
@@ -1143,15 +1256,9 @@ fn start_or_reload_plugin(env: &ForeignFunctionEnv, url: &str) -> Result<()> {
         )
     };
     let cwd = std::env::current_dir().unwrap_or_else(|_| PathBuf::from("."));
-    let url = Url::parse(&url).map_err(|e| anyhow!("Failed to parse url: {}", e))?;
-    let run_plugin_location = RunPluginLocation::parse(url.as_str(), Some(cwd))
+    let run_plugin_or_alias = RunPluginOrAlias::from_url(url, &None, None, Some(cwd))
         .map_err(|e| anyhow!("Failed to parse plugin location: {}", e))?;
-    let run_plugin = RunPlugin {
-        location: run_plugin_location,
-        _allow_exec_host_cmd: false,
-        configuration: PluginUserConfiguration::new(BTreeMap::new()), // TODO: allow passing configuration
-    };
-    let action = Action::StartOrReloadPlugin(run_plugin);
+    let action = Action::StartOrReloadPlugin(run_plugin_or_alias);
     apply_action!(action, error_msg, env);
     Ok(())
 }
@@ -1224,6 +1331,114 @@ fn rename_session(env: &ForeignFunctionEnv, new_session_name: String) {
     apply_action!(action, error_msg, env);
 }
 
+fn disconnect_other_clients(env: &ForeignFunctionEnv) {
+    let _ = env
+        .plugin_env
+        .senders
+        .send_to_server(ServerInstruction::DisconnectAllClientsExcept(
+            env.plugin_env.client_id,
+        ))
+        .context("failed to send disconnect other clients instruction");
+}
+
+fn kill_sessions(session_names: Vec<String>) {
+    for session_name in session_names {
+        let path = &*ZELLIJ_SOCK_DIR.join(&session_name);
+        match LocalSocketStream::connect(path) {
+            Ok(stream) => {
+                let _ = IpcSenderWithContext::new(stream).send(ClientToServerMsg::KillSession);
+            },
+            Err(e) => {
+                log::error!("Failed to kill session {}: {:?}", session_name, e);
+            },
+        };
+    }
+}
+
+fn watch_filesystem(env: &ForeignFunctionEnv) {
+    let _ = env
+        .plugin_env
+        .senders
+        .to_plugin
+        .as_ref()
+        .map(|sender| sender.send(PluginInstruction::WatchFilesystem));
+}
+
+fn dump_session_layout(env: &ForeignFunctionEnv) {
+    let _ = env.plugin_env.senders.to_screen.as_ref().map(|sender| {
+        sender.send(ScreenInstruction::DumpLayoutToPlugin(
+            env.plugin_env.plugin_id,
+        ))
+    });
+}
+
+fn scan_host_folder(env: &ForeignFunctionEnv, folder_to_scan: PathBuf) {
+    if !folder_to_scan.starts_with("/host") {
+        log::error!(
+            "Can only scan files in the /host filesystem, found: {}",
+            folder_to_scan.display()
+        );
+        return;
+    }
+    let plugin_host_folder = env.plugin_env.plugin_cwd.clone();
+    let folder_to_scan = plugin_host_folder.join(folder_to_scan.strip_prefix("/host").unwrap());
+    match folder_to_scan.canonicalize() {
+        Ok(folder_to_scan) => {
+            if !folder_to_scan.starts_with(&plugin_host_folder) {
+                log::error!(
+                    "Can only scan files in the plugin filesystem: {}, found: {}",
+                    plugin_host_folder.display(),
+                    folder_to_scan.display()
+                );
+                return;
+            }
+            let reading_folder = std::fs::read_dir(&folder_to_scan);
+            match reading_folder {
+                Ok(reading_folder) => {
+                    let send_plugin_instructions = env.plugin_env.senders.to_plugin.clone();
+                    let update_target = Some(env.plugin_env.plugin_id);
+                    let client_id = env.plugin_env.client_id;
+                    thread::spawn({
+                        move || {
+                            let mut paths_in_folder = vec![];
+                            for entry in reading_folder {
+                                if let Ok(entry) = entry {
+                                    let entry_metadata = entry.metadata().ok().map(|m| m.into());
+                                    paths_in_folder.push((
+                                        PathBuf::from("/host").join(
+                                            entry.path().strip_prefix(&plugin_host_folder).unwrap(),
+                                        ),
+                                        entry_metadata.into(),
+                                    ));
+                                }
+                            }
+                            let _ = send_plugin_instructions
+                                .ok_or(anyhow!("found no sender to send plugin instruction to"))
+                                .map(|sender| {
+                                    let _ = sender.send(PluginInstruction::Update(vec![(
+                                        update_target,
+                                        Some(client_id),
+                                        Event::FileSystemUpdate(paths_in_folder),
+                                    )]));
+                                })
+                                .non_fatal();
+                        }
+                    });
+                },
+                Err(e) => {
+                    log::error!("Failed to read folder {}: {e}", folder_to_scan.display());
+                },
+            }
+        },
+        Err(e) => {
+            log::error!(
+                "Failed to canonicalize path {folder_to_scan:?} when scanning folder: {:?}",
+                e
+            );
+        },
+    }
+}
+
 // Custom panic handler for plugins.
 //
 // This is called when a panic occurs in a plugin. Since most panics will likely originate in the
@@ -1352,7 +1567,14 @@ fn check_command_permission(
         | PluginCommand::DeleteDeadSession(..)
         | PluginCommand::DeleteAllDeadSessions
         | PluginCommand::RenameSession(..)
-        | PluginCommand::RenameTab(..) => PermissionType::ChangeApplicationState,
+        | PluginCommand::RenameTab(..)
+        | PluginCommand::DisconnectOtherClients
+        | PluginCommand::KillSessions(..) => PermissionType::ChangeApplicationState,
+        PluginCommand::UnblockCliPipeInput(..)
+        | PluginCommand::BlockCliPipeInput(..)
+        | PluginCommand::CliPipeOutput(..) => PermissionType::ReadCliPipes,
+        PluginCommand::MessageToPlugin(..) => PermissionType::MessageAndLaunchOtherPlugins,
+        PluginCommand::DumpSessionLayout => PermissionType::ReadApplicationState,
         _ => return (PermissionStatus::Granted, None),
     };
 
