@@ -2,7 +2,7 @@ use crate::background_jobs::BackgroundJob;
 use crate::terminal_bytes::TerminalBytes;
 use crate::{
     panes::PaneId,
-    plugins::PluginInstruction,
+    plugins::{PluginId, PluginInstruction},
     screen::ScreenInstruction,
     session_layout_metadata::SessionLayoutMetadata,
     thread_bus::{Bus, ThreadSenders},
@@ -13,14 +13,12 @@ use std::{collections::HashMap, os::unix::io::RawFd, path::PathBuf};
 use zellij_utils::nix::unistd::Pid;
 use zellij_utils::{
     async_std,
+    data::FloatingPaneCoordinates,
     errors::prelude::*,
     errors::{ContextType, PtyContext},
     input::{
         command::{RunCommand, TerminalAction},
-        layout::{
-            FloatingPaneLayout, Layout, PluginUserConfiguration, Run, RunPlugin, RunPluginLocation,
-            TiledPaneLayout,
-        },
+        layout::{FloatingPaneLayout, Layout, Run, RunPluginOrAlias, TiledPaneLayout},
     },
     pane_size::Size,
     session_serialization,
@@ -43,6 +41,7 @@ pub enum PtyInstruction {
         Option<TerminalAction>,
         Option<bool>,
         Option<String>,
+        Option<FloatingPaneCoordinates>,
         ClientTabIndexOrPaneId,
     ), // bool (if Some) is
     // should_float, String is an optional pane name
@@ -60,8 +59,8 @@ pub enum PtyInstruction {
         Option<TerminalAction>,
         Option<TiledPaneLayout>,
         Vec<FloatingPaneLayout>,
-        usize,                                                           // tab_index
-        HashMap<(RunPluginLocation, PluginUserConfiguration), Vec<u32>>, // plugin_ids
+        usize,                               // tab_index
+        HashMap<RunPluginOrAlias, Vec<u32>>, // plugin_ids
         ClientId,
     ), // the String is the tab name
     ClosePane(PaneId),
@@ -78,16 +77,20 @@ pub enum PtyInstruction {
         ClientTabIndexOrPaneId,
     ), // String is an optional pane name
     DumpLayout(SessionLayoutMetadata, ClientId),
+    DumpLayoutToPlugin(SessionLayoutMetadata, PluginId),
     LogLayoutToHd(SessionLayoutMetadata),
     FillPluginCwd(
         Option<bool>,   // should float
         bool,           // should be opened in place
         Option<String>, // pane title
-        RunPlugin,
+        RunPluginOrAlias,
         usize,          // tab index
         Option<PaneId>, // pane id to replace if this is to be opened "in-place"
         ClientId,
         Size,
+        bool,            // skip cache
+        Option<PathBuf>, // if Some, will not fill cwd but just forward the message
+        Option<FloatingPaneCoordinates>,
     ),
     Exit,
 }
@@ -108,6 +111,7 @@ impl From<&PtyInstruction> for PtyContext {
             PtyInstruction::DropToShellInPane { .. } => PtyContext::DropToShellInPane,
             PtyInstruction::SpawnInPlaceTerminal(..) => PtyContext::SpawnInPlaceTerminal,
             PtyInstruction::DumpLayout(..) => PtyContext::DumpLayout,
+            PtyInstruction::DumpLayoutToPlugin(..) => PtyContext::DumpLayoutToPlugin,
             PtyInstruction::LogLayoutToHd(..) => PtyContext::LogLayoutToHd,
             PtyInstruction::FillPluginCwd(..) => PtyContext::FillPluginCwd,
             PtyInstruction::Exit => PtyContext::Exit,
@@ -133,6 +137,7 @@ pub(crate) fn pty_thread_main(mut pty: Pty, layout: Box<Layout>) -> Result<()> {
                 terminal_action,
                 should_float,
                 name,
+                floating_pane_coordinates,
                 client_or_tab_index,
             ) => {
                 let err_context =
@@ -174,6 +179,7 @@ pub(crate) fn pty_thread_main(mut pty: Pty, layout: Box<Layout>) -> Result<()> {
                                 should_float,
                                 hold_for_command,
                                 invoked_with,
+                                floating_pane_coordinates,
                                 client_or_tab_index,
                             ))
                             .with_context(err_context)?;
@@ -190,6 +196,7 @@ pub(crate) fn pty_thread_main(mut pty: Pty, layout: Box<Layout>) -> Result<()> {
                                         should_float,
                                         hold_for_command,
                                         invoked_with,
+                                        floating_pane_coordinates,
                                         client_or_tab_index,
                                     ))
                                     .with_context(err_context)?;
@@ -625,6 +632,18 @@ pub(crate) fn pty_thread_main(mut pty: Pty, layout: Box<Layout>) -> Result<()> {
                     },
                 }
             },
+            PtyInstruction::DumpLayoutToPlugin(mut session_layout_metadata, plugin_id) => {
+                let err_context = || format!("Failed to dump layout");
+                pty.populate_session_layout_metadata(&mut session_layout_metadata);
+                pty.bus
+                    .senders
+                    .send_to_plugin(PluginInstruction::DumpLayoutToPlugin(
+                        session_layout_metadata,
+                        plugin_id,
+                    ))
+                    .with_context(err_context)
+                    .non_fatal();
+            },
             PtyInstruction::LogLayoutToHd(mut session_layout_metadata) => {
                 let err_context = || format!("Failed to dump layout");
                 pty.populate_session_layout_metadata(&mut session_layout_metadata);
@@ -653,6 +672,9 @@ pub(crate) fn pty_thread_main(mut pty: Pty, layout: Box<Layout>) -> Result<()> {
                 pane_id_to_replace,
                 client_id,
                 size,
+                skip_cache,
+                cwd,
+                floating_pane_coordinates,
             ) => {
                 pty.fill_plugin_cwd(
                     should_float,
@@ -663,6 +685,9 @@ pub(crate) fn pty_thread_main(mut pty: Pty, layout: Box<Layout>) -> Result<()> {
                     pane_id_to_replace,
                     client_id,
                     size,
+                    skip_cache,
+                    cwd,
+                    floating_pane_coordinates,
                 )?;
             },
             PtyInstruction::Exit => break,
@@ -860,7 +885,7 @@ impl Pty {
         layout: TiledPaneLayout,
         floating_panes_layout: Vec<FloatingPaneLayout>,
         default_shell: Option<TerminalAction>,
-        plugin_ids: HashMap<(RunPluginLocation, PluginUserConfiguration), Vec<u32>>,
+        plugin_ids: HashMap<RunPluginOrAlias, Vec<u32>>,
         tab_index: usize,
         client_id: ClientId,
     ) -> Result<()> {
@@ -1323,26 +1348,38 @@ impl Pty {
         should_float: Option<bool>,
         should_open_in_place: bool, // should be opened in place
         pane_title: Option<String>, // pane title
-        run: RunPlugin,
+        mut run: RunPluginOrAlias,
         tab_index: usize,                   // tab index
         pane_id_to_replace: Option<PaneId>, // pane id to replace if this is to be opened "in-place"
         client_id: ClientId,
         size: Size,
+        skip_cache: bool,
+        cwd: Option<PathBuf>,
+        // left here for historical and potential future reasons since we might change the ordering
+        // of the pipeline between threads and end up needing to forward this
+        _floating_pane_coordinates: Option<FloatingPaneCoordinates>,
     ) -> Result<()> {
-        let cwd = self
-            .active_panes
-            .get(&client_id)
-            .and_then(|pane| match pane {
-                PaneId::Plugin(..) => None,
-                PaneId::Terminal(id) => self.id_to_child_pid.get(id),
-            })
-            .and_then(|&id| {
-                self.bus
-                    .os_input
-                    .as_ref()
-                    .and_then(|input| input.get_cwd(Pid::from_raw(id)))
-            });
+        let get_focused_cwd = || {
+            self.active_panes
+                .get(&client_id)
+                .and_then(|pane| match pane {
+                    PaneId::Plugin(..) => None,
+                    PaneId::Terminal(id) => self.id_to_child_pid.get(id),
+                })
+                .and_then(|&id| {
+                    self.bus
+                        .os_input
+                        .as_ref()
+                        .and_then(|input| input.get_cwd(Pid::from_raw(id)))
+                })
+        };
 
+        let cwd = cwd.or_else(get_focused_cwd);
+
+        if let RunPluginOrAlias::Alias(alias) = &mut run {
+            let cwd = get_focused_cwd();
+            alias.set_caller_cwd_if_not_set(cwd);
+        }
         self.bus.senders.send_to_plugin(PluginInstruction::Load(
             should_float,
             should_open_in_place,
@@ -1353,6 +1390,7 @@ impl Pty {
             client_id,
             size,
             cwd,
+            skip_cache,
         ))?;
         Ok(())
     }
