@@ -1,4 +1,4 @@
-use std::collections::VecDeque;
+use std::collections::{BTreeMap, HashSet, VecDeque};
 use std::sync::{Arc, RwLock};
 
 use crate::thread_bus::ThreadSenders;
@@ -10,6 +10,7 @@ use crate::{
     screen::ScreenInstruction,
     ServerInstruction, SessionMetaData, SessionState,
 };
+use uuid::Uuid;
 use zellij_utils::{
     channels::SenderWithContext,
     data::{Direction, Event, PluginCapabilities, ResizeStrategy},
@@ -36,6 +37,7 @@ pub(crate) fn route_action(
     client_attributes: ClientAttributes,
     default_shell: Option<TerminalAction>,
     default_layout: Box<Layout>,
+    mut seen_cli_pipes: Option<&mut HashSet<String>>,
 ) -> Result<bool> {
     let mut should_break = false;
     let err_context = || format!("failed to route action for client {client_id}");
@@ -267,6 +269,7 @@ pub(crate) fn route_action(
                     shell,
                     None,
                     name,
+                    None,
                     ClientTabIndexOrPaneId::ClientId(client_id),
                 ),
             };
@@ -279,6 +282,7 @@ pub(crate) fn route_action(
             split_direction,
             should_float,
             should_open_in_place,
+            floating_pane_coordinates,
         ) => {
             let title = format!("Editing: {}", path_to_file.display());
             let open_file = TerminalAction::OpenFile(path_to_file, line_number, cwd);
@@ -318,6 +322,7 @@ pub(crate) fn route_action(
                     Some(open_file),
                     Some(should_float),
                     Some(title),
+                    floating_pane_coordinates,
                     ClientTabIndexOrPaneId::ClientId(client_id),
                 ),
             };
@@ -340,7 +345,7 @@ pub(crate) fn route_action(
                 )))
                 .with_context(err_context)?;
         },
-        Action::NewFloatingPane(run_command, name) => {
+        Action::NewFloatingPane(run_command, name, floating_pane_coordinates) => {
             let should_float = true;
             let run_cmd = run_command
                 .map(|cmd| TerminalAction::RunCommand(cmd.into()))
@@ -350,6 +355,7 @@ pub(crate) fn route_action(
                     run_cmd,
                     Some(should_float),
                     name,
+                    floating_pane_coordinates,
                     ClientTabIndexOrPaneId::ClientId(client_id),
                 ))
                 .with_context(err_context)?;
@@ -402,6 +408,7 @@ pub(crate) fn route_action(
                     run_cmd,
                     Some(should_float),
                     name,
+                    None,
                     ClientTabIndexOrPaneId::ClientId(client_id),
                 ),
             };
@@ -450,6 +457,7 @@ pub(crate) fn route_action(
                     run_cmd,
                     None,
                     None,
+                    None,
                     ClientTabIndexOrPaneId::ClientId(client_id),
                 ),
             };
@@ -533,6 +541,16 @@ pub(crate) fn route_action(
                 .send_to_screen(ScreenInstruction::UndoRenameTab(client_id))
                 .with_context(err_context)?;
         },
+        Action::MoveTab(direction) => {
+            let screen_instr = match direction {
+                Direction::Left => ScreenInstruction::MoveTabLeft(client_id),
+                Direction::Right => ScreenInstruction::MoveTabRight(client_id),
+                _ => return Ok(false),
+            };
+            senders
+                .send_to_screen(screen_instr)
+                .with_context(err_context)?;
+        },
         Action::Quit => {
             senders
                 .send_to_server(ServerInstruction::ClientExit(client_id))
@@ -658,25 +676,36 @@ pub(crate) fn route_action(
                 .send_to_screen(ScreenInstruction::QueryTabNames(client_id))
                 .with_context(err_context)?;
         },
-        Action::NewTiledPluginPane(run_plugin, name) => {
+        Action::NewTiledPluginPane(run_plugin, name, skip_cache, cwd) => {
             senders
                 .send_to_screen(ScreenInstruction::NewTiledPluginPane(
-                    run_plugin, name, client_id,
+                    run_plugin, name, skip_cache, cwd, client_id,
                 ))
                 .with_context(err_context)?;
         },
-        Action::NewFloatingPluginPane(run_plugin, name) => {
+        Action::NewFloatingPluginPane(
+            run_plugin,
+            name,
+            skip_cache,
+            cwd,
+            floating_pane_coordinates,
+        ) => {
             senders
                 .send_to_screen(ScreenInstruction::NewFloatingPluginPane(
-                    run_plugin, name, client_id,
+                    run_plugin,
+                    name,
+                    skip_cache,
+                    cwd,
+                    floating_pane_coordinates,
+                    client_id,
                 ))
                 .with_context(err_context)?;
         },
-        Action::NewInPlacePluginPane(run_plugin, name) => {
+        Action::NewInPlacePluginPane(run_plugin, name, skip_cache) => {
             if let Some(pane_id) = pane_id {
                 senders
                     .send_to_screen(ScreenInstruction::NewInPlacePluginPane(
-                        run_plugin, name, pane_id, client_id,
+                        run_plugin, name, pane_id, skip_cache, client_id,
                     ))
                     .with_context(err_context)?;
             } else {
@@ -693,6 +722,7 @@ pub(crate) fn route_action(
             should_float,
             move_to_focused_tab,
             should_open_in_place,
+            skip_cache,
         ) => {
             senders
                 .send_to_screen(ScreenInstruction::LaunchOrFocusPlugin(
@@ -701,17 +731,20 @@ pub(crate) fn route_action(
                     move_to_focused_tab,
                     should_open_in_place,
                     pane_id,
+                    skip_cache,
                     client_id,
                 ))
                 .with_context(err_context)?;
         },
-        Action::LaunchPlugin(run_plugin, should_float, should_open_in_place) => {
+        Action::LaunchPlugin(run_plugin, should_float, should_open_in_place, skip_cache, cwd) => {
             senders
                 .send_to_screen(ScreenInstruction::LaunchPlugin(
                     run_plugin,
                     should_float,
                     should_open_in_place,
                     pane_id,
+                    skip_cache,
+                    cwd,
                     client_id,
                 ))
                 .with_context(err_context)?;
@@ -800,6 +833,99 @@ pub(crate) fn route_action(
                 .send_to_screen(ScreenInstruction::RenameSession(name, client_id))
                 .with_context(err_context)?;
         },
+        Action::CliPipe {
+            pipe_id,
+            mut name,
+            payload,
+            plugin,
+            args,
+            configuration,
+            floating,
+            in_place,
+            skip_cache,
+            cwd,
+            pane_title,
+            ..
+        } => {
+            if let Some(seen_cli_pipes) = seen_cli_pipes.as_mut() {
+                if !seen_cli_pipes.contains(&pipe_id) {
+                    seen_cli_pipes.insert(pipe_id.clone());
+                    senders
+                        .send_to_server(ServerInstruction::AssociatePipeWithClient {
+                            pipe_id: pipe_id.clone(),
+                            client_id,
+                        })
+                        .with_context(err_context)?;
+                }
+            }
+            if let Some(name) = name.take() {
+                let should_open_in_place = in_place.unwrap_or(false);
+                if should_open_in_place && pane_id.is_none() {
+                    log::error!("Was asked to open a new plugin in-place, but cannot identify the pane id... is the ZELLIJ_PANE_ID variable set?");
+                }
+                let pane_id_to_replace = if should_open_in_place { pane_id } else { None };
+                senders
+                    .send_to_plugin(PluginInstruction::CliPipe {
+                        pipe_id,
+                        name,
+                        payload,
+                        plugin,
+                        args,
+                        configuration,
+                        floating,
+                        pane_id_to_replace,
+                        cwd,
+                        pane_title,
+                        skip_cache,
+                        cli_client_id: client_id,
+                    })
+                    .with_context(err_context)?;
+            } else {
+                log::error!("Message must have a name");
+            }
+        },
+        Action::KeybindPipe {
+            mut name,
+            payload,
+            plugin,
+            args,
+            mut configuration,
+            floating,
+            in_place,
+            skip_cache,
+            cwd,
+            pane_title,
+            launch_new,
+            ..
+        } => {
+            if let Some(name) = name.take() {
+                let should_open_in_place = in_place.unwrap_or(false);
+                let pane_id_to_replace = if should_open_in_place { pane_id } else { None };
+                if launch_new {
+                    // we do this to make sure the plugin is unique (has a unique configuration parameter)
+                    configuration
+                        .get_or_insert_with(BTreeMap::new)
+                        .insert("_zellij_id".to_owned(), Uuid::new_v4().to_string());
+                }
+                senders
+                    .send_to_plugin(PluginInstruction::KeybindPipe {
+                        name,
+                        payload,
+                        plugin,
+                        args,
+                        configuration,
+                        floating,
+                        pane_id_to_replace,
+                        cwd,
+                        pane_title,
+                        skip_cache,
+                        cli_client_id: client_id,
+                    })
+                    .with_context(err_context)?;
+            } else {
+                log::error!("Message must have a name");
+            }
+        },
     }
     Ok(should_break)
 }
@@ -830,13 +956,14 @@ pub(crate) fn route_thread_main(
 ) -> Result<()> {
     let mut retry_queue = VecDeque::new();
     let err_context = || format!("failed to handle instruction for client {client_id}");
+    let mut seen_cli_pipes = HashSet::new();
     'route_loop: loop {
         match receiver.recv() {
             Some((instruction, err_ctx)) => {
                 err_ctx.update_thread_ctx();
                 let rlocked_sessions = session_data.read().to_anyhow().with_context(err_context)?;
-                let handle_instruction = |instruction: ClientToServerMsg,
-                                          mut retry_queue: Option<
+                let mut handle_instruction = |instruction: ClientToServerMsg,
+                                              mut retry_queue: Option<
                     &mut VecDeque<ClientToServerMsg>,
                 >|
                  -> Result<bool> {
@@ -865,6 +992,7 @@ pub(crate) fn route_thread_main(
                                     rlocked_sessions.client_attributes.clone(),
                                     rlocked_sessions.default_shell.clone(),
                                     rlocked_sessions.layout.clone(),
+                                    Some(&mut seen_cli_pipes),
                                 )? {
                                     should_break = true;
                                 }
@@ -938,15 +1066,15 @@ pub(crate) fn route_thread_main(
                             cli_args,
                             opts,
                             layout,
-                            plugin_config,
+                            plugin_aliases,
                         ) => {
                             let new_client_instruction = ServerInstruction::NewClient(
                                 client_attributes,
                                 cli_args,
                                 opts,
                                 layout,
+                                plugin_aliases,
                                 client_id,
-                                plugin_config,
                             );
                             to_server
                                 .send(new_client_instruction)
