@@ -2,12 +2,13 @@ pub use super::generated_api::api::{
     action::{
         action::OptionalPayload, Action as ProtobufAction, ActionName as ProtobufActionName,
         DumpScreenPayload, EditFilePayload, GoToTabNamePayload, IdAndName,
-        LaunchOrFocusPluginPayload, MovePanePayload, NameAndValue as ProtobufNameAndValue,
-        NewFloatingPanePayload, NewPanePayload, NewPluginPanePayload, NewTiledPanePayload,
-        PaneIdAndShouldFloat, PluginConfiguration as ProtobufPluginConfiguration,
-        Position as ProtobufPosition, RunCommandAction as ProtobufRunCommandAction,
-        ScrollAtPayload, SearchDirection as ProtobufSearchDirection,
-        SearchOption as ProtobufSearchOption, SwitchToModePayload, WriteCharsPayload, WritePayload,
+        LaunchOrFocusPluginPayload, MovePanePayload, MoveTabDirection as ProtobufMoveTabDirection,
+        NameAndValue as ProtobufNameAndValue, NewFloatingPanePayload, NewPanePayload,
+        NewPluginPanePayload, NewTiledPanePayload, PaneIdAndShouldFloat,
+        PluginConfiguration as ProtobufPluginConfiguration, Position as ProtobufPosition,
+        RunCommandAction as ProtobufRunCommandAction, ScrollAtPayload,
+        SearchDirection as ProtobufSearchDirection, SearchOption as ProtobufSearchOption,
+        SwitchToModePayload, WriteCharsPayload, WritePayload,
     },
     input_mode::InputMode as ProtobufInputMode,
     resize::{Resize as ProtobufResize, ResizeDirection as ProtobufResizeDirection},
@@ -17,9 +18,10 @@ use crate::errors::prelude::*;
 use crate::input::actions::Action;
 use crate::input::actions::{SearchDirection, SearchOption};
 use crate::input::command::RunCommandAction;
-use crate::input::layout::{PluginUserConfiguration, RunPlugin, RunPluginLocation};
+use crate::input::layout::{
+    PluginUserConfiguration, RunPlugin, RunPluginLocation, RunPluginOrAlias,
+};
 use crate::position::Position;
-use url::Url;
 
 use std::collections::BTreeMap;
 use std::convert::TryFrom;
@@ -240,6 +242,7 @@ impl TryFrom<ProtobufAction> for Action {
                         direction,
                         should_float,
                         should_be_in_place,
+                        None,
                     ))
                 },
                 _ => Err("Wrong payload for Action::NewPane"),
@@ -249,9 +252,13 @@ impl TryFrom<ProtobufAction> for Action {
                     if let Some(payload) = payload.command {
                         let pane_name = payload.pane_name.clone();
                         let run_command_action: RunCommandAction = payload.try_into()?;
-                        Ok(Action::NewFloatingPane(Some(run_command_action), pane_name))
+                        Ok(Action::NewFloatingPane(
+                            Some(run_command_action),
+                            pane_name,
+                            None,
+                        ))
                     } else {
-                        Ok(Action::NewFloatingPane(None, None))
+                        Ok(Action::NewFloatingPane(None, None, None))
                     }
                 },
                 _ => Err("Wrong payload for Action::NewFloatingPane"),
@@ -353,6 +360,15 @@ impl TryFrom<ProtobufAction> for Action {
                 Some(_) => Err("UndoRenameTab should not have a payload"),
                 None => Ok(Action::UndoRenameTab),
             },
+            Some(ProtobufActionName::MoveTab) => match protobuf_action.optional_payload {
+                Some(OptionalPayload::MoveTabPayload(move_tab_payload)) => {
+                    let direction: Direction = ProtobufMoveTabDirection::from_i32(move_tab_payload)
+                        .ok_or("Malformed move tab direction for Action::MoveTab")?
+                        .try_into()?;
+                    Ok(Action::MoveTab(direction))
+                },
+                _ => Err("Wrong payload for Action::MoveTab"),
+            },
             Some(ProtobufActionName::Run) => match protobuf_action.optional_payload {
                 Some(OptionalPayload::RunPayload(run_command_action)) => {
                     let run_command_action = run_command_action.try_into()?;
@@ -388,26 +404,27 @@ impl TryFrom<ProtobufAction> for Action {
             Some(ProtobufActionName::LaunchOrFocusPlugin) => {
                 match protobuf_action.optional_payload {
                     Some(OptionalPayload::LaunchOrFocusPluginPayload(payload)) => {
-                        let run_plugin_location =
-                            RunPluginLocation::parse(&payload.plugin_url, None)
-                                .map_err(|_| "Malformed LaunchOrFocusPlugin payload")?;
                         let configuration: PluginUserConfiguration = payload
                             .plugin_configuration
                             .and_then(|p| PluginUserConfiguration::try_from(p).ok())
                             .unwrap_or_default();
-                        let run_plugin = RunPlugin {
-                            _allow_exec_host_cmd: false,
-                            location: run_plugin_location,
-                            configuration,
-                        };
+                        let run_plugin_or_alias = RunPluginOrAlias::from_url(
+                            &payload.plugin_url.as_str(),
+                            &Some(configuration.inner().clone()),
+                            None,
+                            None,
+                        )
+                        .map_err(|_| "Malformed LaunchOrFocusPlugin payload")?;
                         let should_float = payload.should_float;
                         let move_to_focused_tab = payload.move_to_focused_tab;
                         let should_open_in_place = payload.should_open_in_place;
+                        let skip_plugin_cache = payload.skip_plugin_cache;
                         Ok(Action::LaunchOrFocusPlugin(
-                            run_plugin,
+                            run_plugin_or_alias,
                             should_float,
                             move_to_focused_tab,
                             should_open_in_place,
+                            skip_plugin_cache,
                         ))
                     },
                     _ => Err("Wrong payload for Action::LaunchOrFocusPlugin"),
@@ -415,26 +432,28 @@ impl TryFrom<ProtobufAction> for Action {
             },
             Some(ProtobufActionName::LaunchPlugin) => match protobuf_action.optional_payload {
                 Some(OptionalPayload::LaunchOrFocusPluginPayload(payload)) => {
-                    let run_plugin_location =
-                        RunPluginLocation::parse(&payload.plugin_url, None)
-                            .map_err(|_| "Malformed LaunchOrFocusPlugin payload")?;
                     let configuration: PluginUserConfiguration = payload
                         .plugin_configuration
                         .and_then(|p| PluginUserConfiguration::try_from(p).ok())
                         .unwrap_or_default();
-                    let run_plugin = RunPlugin {
-                        _allow_exec_host_cmd: false,
-                        location: run_plugin_location,
-                        configuration,
-                    };
+                    let run_plugin_or_alias = RunPluginOrAlias::from_url(
+                        &payload.plugin_url.as_str(),
+                        &Some(configuration.inner().clone()),
+                        None,
+                        None,
+                    )
+                    .map_err(|_| "Malformed LaunchOrFocusPlugin payload")?;
                     let should_float = payload.should_float;
                     let _move_to_focused_tab = payload.move_to_focused_tab; // not actually used in
                                                                             // this action
                     let should_open_in_place = payload.should_open_in_place;
+                    let skip_plugin_cache = payload.skip_plugin_cache;
                     Ok(Action::LaunchPlugin(
-                        run_plugin,
+                        run_plugin_or_alias,
                         should_float,
                         should_open_in_place,
+                        skip_plugin_cache,
+                        None,
                     ))
                 },
                 _ => Err("Wrong payload for Action::LaunchOrFocusPlugin"),
@@ -533,13 +552,20 @@ impl TryFrom<ProtobufAction> for Action {
                         let run_plugin_location =
                             RunPluginLocation::parse(&payload.plugin_url, None)
                                 .map_err(|_| "Malformed NewTiledPluginPane payload")?;
-                        let run_plugin = RunPlugin {
+                        let run_plugin = RunPluginOrAlias::RunPlugin(RunPlugin {
                             location: run_plugin_location,
                             _allow_exec_host_cmd: false,
                             configuration: PluginUserConfiguration::default(),
-                        };
+                            ..Default::default()
+                        });
                         let pane_name = payload.pane_name;
-                        Ok(Action::NewTiledPluginPane(run_plugin, pane_name))
+                        let skip_plugin_cache = payload.skip_plugin_cache;
+                        Ok(Action::NewTiledPluginPane(
+                            run_plugin,
+                            pane_name,
+                            skip_plugin_cache,
+                            None,
+                        ))
                     },
                     _ => Err("Wrong payload for Action::NewTiledPluginPane"),
                 }
@@ -550,13 +576,21 @@ impl TryFrom<ProtobufAction> for Action {
                         let run_plugin_location =
                             RunPluginLocation::parse(&payload.plugin_url, None)
                                 .map_err(|_| "Malformed NewTiledPluginPane payload")?;
-                        let run_plugin = RunPlugin {
+                        let run_plugin = RunPluginOrAlias::RunPlugin(RunPlugin {
                             location: run_plugin_location,
                             _allow_exec_host_cmd: false,
                             configuration: PluginUserConfiguration::default(),
-                        };
+                            ..Default::default()
+                        });
                         let pane_name = payload.pane_name;
-                        Ok(Action::NewFloatingPluginPane(run_plugin, pane_name))
+                        let skip_plugin_cache = payload.skip_plugin_cache;
+                        Ok(Action::NewFloatingPluginPane(
+                            run_plugin,
+                            pane_name,
+                            skip_plugin_cache,
+                            None,
+                            None,
+                        ))
                     },
                     _ => Err("Wrong payload for Action::MiddleClick"),
                 }
@@ -564,14 +598,11 @@ impl TryFrom<ProtobufAction> for Action {
             Some(ProtobufActionName::StartOrReloadPlugin) => {
                 match protobuf_action.optional_payload {
                     Some(OptionalPayload::StartOrReloadPluginPayload(payload)) => {
-                        let run_plugin_location = RunPluginLocation::parse(&payload, None)
-                            .map_err(|_| "Malformed StartOrReloadPluginPayload payload")?;
-                        let run_plugin = RunPlugin {
-                            _allow_exec_host_cmd: false,
-                            location: run_plugin_location,
-                            configuration: PluginUserConfiguration::default(),
-                        };
-                        Ok(Action::StartOrReloadPlugin(run_plugin))
+                        let run_plugin_or_alias =
+                            RunPluginOrAlias::from_url(&payload.as_str(), &None, None, None)
+                                .map_err(|_| "Malformed LaunchOrFocusPlugin payload")?;
+
+                        Ok(Action::StartOrReloadPlugin(run_plugin_or_alias))
                     },
                     _ => Err("Wrong payload for Action::StartOrReloadPlugin"),
                 }
@@ -658,6 +689,23 @@ impl TryFrom<ProtobufAction> for Action {
                 },
                 _ => Err("Wrong payload for Action::RenameSession"),
             },
+            Some(ProtobufActionName::KeybindPipe) => match protobuf_action.optional_payload {
+                Some(_) => Err("KeybindPipe should not have a payload"),
+                // TODO: at some point we might want to support a payload here
+                None => Ok(Action::KeybindPipe {
+                    name: None,
+                    payload: None,
+                    args: None,
+                    plugin: None,
+                    configuration: None,
+                    launch_new: false,
+                    skip_cache: false,
+                    floating: None,
+                    in_place: None,
+                    cwd: None,
+                    pane_title: None,
+                }),
+            },
             _ => Err("Unknown Action"),
         }
     }
@@ -857,6 +905,7 @@ impl TryFrom<Action> for ProtobufAction {
                 direction,
                 should_float,
                 _should_be_in_place,
+                _floating_pane_coordinates,
             ) => {
                 let file_to_edit = path_to_file.display().to_string();
                 let cwd = cwd.map(|cwd| cwd.display().to_string());
@@ -875,7 +924,7 @@ impl TryFrom<Action> for ProtobufAction {
                     })),
                 })
             },
-            Action::NewFloatingPane(run_command_action, pane_name) => {
+            Action::NewFloatingPane(run_command_action, pane_name, _coordinates) => {
                 let command = run_command_action.and_then(|r| {
                     let mut protobuf_run_command_action: ProtobufRunCommandAction =
                         r.try_into().ok()?;
@@ -970,6 +1019,13 @@ impl TryFrom<Action> for ProtobufAction {
                 name: ProtobufActionName::UndoRenameTab as i32,
                 optional_payload: None,
             }),
+            Action::MoveTab(direction) => {
+                let direction: ProtobufMoveTabDirection = direction.try_into()?;
+                Ok(ProtobufAction {
+                    name: ProtobufActionName::MoveTab as i32,
+                    optional_payload: Some(OptionalPayload::MoveTabPayload(direction as i32)),
+                })
+            },
             Action::Run(run_command_action) => {
                 let run_command_action: ProtobufRunCommandAction = run_command_action.try_into()?;
                 Ok(ProtobufAction {
@@ -1003,36 +1059,45 @@ impl TryFrom<Action> for ProtobufAction {
                 })
             },
             Action::LaunchOrFocusPlugin(
-                run_plugin,
+                run_plugin_or_alias,
                 should_float,
                 move_to_focused_tab,
                 should_open_in_place,
+                skip_plugin_cache,
             ) => {
-                let url: Url = Url::from(&run_plugin.location);
+                let configuration = run_plugin_or_alias.get_configuration().unwrap_or_default();
                 Ok(ProtobufAction {
                     name: ProtobufActionName::LaunchOrFocusPlugin as i32,
                     optional_payload: Some(OptionalPayload::LaunchOrFocusPluginPayload(
                         LaunchOrFocusPluginPayload {
-                            plugin_url: url.into(),
+                            plugin_url: run_plugin_or_alias.location_string(),
                             should_float,
                             move_to_focused_tab,
                             should_open_in_place,
-                            plugin_configuration: Some(run_plugin.configuration.try_into()?),
+                            plugin_configuration: Some(configuration.try_into()?),
+                            skip_plugin_cache,
                         },
                     )),
                 })
             },
-            Action::LaunchPlugin(run_plugin, should_float, should_open_in_place) => {
-                let url: Url = Url::from(&run_plugin.location);
+            Action::LaunchPlugin(
+                run_plugin_or_alias,
+                should_float,
+                should_open_in_place,
+                skip_plugin_cache,
+                _cwd,
+            ) => {
+                let configuration = run_plugin_or_alias.get_configuration().unwrap_or_default();
                 Ok(ProtobufAction {
                     name: ProtobufActionName::LaunchPlugin as i32,
                     optional_payload: Some(OptionalPayload::LaunchOrFocusPluginPayload(
                         LaunchOrFocusPluginPayload {
-                            plugin_url: url.into(),
+                            plugin_url: run_plugin_or_alias.location_string(),
                             should_float,
                             move_to_focused_tab: false,
                             should_open_in_place,
-                            plugin_configuration: Some(run_plugin.configuration.try_into()?),
+                            plugin_configuration: Some(configuration.try_into()?),
+                            skip_plugin_cache,
                         },
                     )),
                 })
@@ -1115,39 +1180,40 @@ impl TryFrom<Action> for ProtobufAction {
                 name: ProtobufActionName::QueryTabNames as i32,
                 optional_payload: None,
             }),
-            Action::NewTiledPluginPane(run_plugin, pane_name) => {
-                let plugin_url: Url = Url::from(&run_plugin.location);
+            Action::NewTiledPluginPane(run_plugin, pane_name, skip_plugin_cache, _cwd) => {
                 Ok(ProtobufAction {
                     name: ProtobufActionName::NewTiledPluginPane as i32,
                     optional_payload: Some(OptionalPayload::NewTiledPluginPanePayload(
                         NewPluginPanePayload {
-                            plugin_url: plugin_url.into(),
+                            plugin_url: run_plugin.location_string(),
                             pane_name,
+                            skip_plugin_cache,
                         },
                     )),
                 })
             },
-            Action::NewFloatingPluginPane(run_plugin, pane_name) => {
-                let plugin_url: Url = Url::from(&run_plugin.location);
-                Ok(ProtobufAction {
-                    name: ProtobufActionName::NewFloatingPluginPane as i32,
-                    optional_payload: Some(OptionalPayload::NewFloatingPluginPanePayload(
-                        NewPluginPanePayload {
-                            plugin_url: plugin_url.into(),
-                            pane_name,
-                        },
-                    )),
-                })
-            },
-            Action::StartOrReloadPlugin(run_plugin) => {
-                let plugin_url: Url = Url::from(&run_plugin.location);
-                Ok(ProtobufAction {
-                    name: ProtobufActionName::StartOrReloadPlugin as i32,
-                    optional_payload: Some(OptionalPayload::StartOrReloadPluginPayload(
-                        plugin_url.into(),
-                    )),
-                })
-            },
+            Action::NewFloatingPluginPane(
+                run_plugin,
+                pane_name,
+                skip_plugin_cache,
+                _cwd,
+                _coordinates,
+            ) => Ok(ProtobufAction {
+                name: ProtobufActionName::NewFloatingPluginPane as i32,
+                optional_payload: Some(OptionalPayload::NewFloatingPluginPanePayload(
+                    NewPluginPanePayload {
+                        plugin_url: run_plugin.location_string(),
+                        pane_name,
+                        skip_plugin_cache,
+                    },
+                )),
+            }),
+            Action::StartOrReloadPlugin(run_plugin) => Ok(ProtobufAction {
+                name: ProtobufActionName::StartOrReloadPlugin as i32,
+                optional_payload: Some(OptionalPayload::StartOrReloadPluginPayload(
+                    run_plugin.location_string(),
+                )),
+            }),
             Action::CloseTerminalPane(terminal_pane_id) => Ok(ProtobufAction {
                 name: ProtobufActionName::CloseTerminalPane as i32,
                 optional_payload: Some(OptionalPayload::CloseTerminalPanePayload(terminal_pane_id)),
@@ -1215,6 +1281,10 @@ impl TryFrom<Action> for ProtobufAction {
                 name: ProtobufActionName::RenameSession as i32,
                 optional_payload: Some(OptionalPayload::RenameSessionPayload(session_name)),
             }),
+            Action::KeybindPipe { .. } => Ok(ProtobufAction {
+                name: ProtobufActionName::KeybindPipe as i32,
+                optional_payload: None,
+            }),
             Action::NoOp
             | Action::Confirm
             | Action::NewInPlacePane(..)
@@ -1222,6 +1292,7 @@ impl TryFrom<Action> for ProtobufAction {
             | Action::Deny
             | Action::Copy
             | Action::DumpLayout
+            | Action::CliPipe { .. }
             | Action::SkipConfirm(..) => Err("Unsupported action"),
         }
     }
@@ -1269,6 +1340,29 @@ impl TryFrom<SearchDirection> for ProtobufSearchDirection {
     }
 }
 
+impl TryFrom<ProtobufMoveTabDirection> for Direction {
+    type Error = &'static str;
+    fn try_from(
+        protobuf_move_tab_direction: ProtobufMoveTabDirection,
+    ) -> Result<Self, &'static str> {
+        match protobuf_move_tab_direction {
+            ProtobufMoveTabDirection::Left => Ok(Direction::Left),
+            ProtobufMoveTabDirection::Right => Ok(Direction::Right),
+        }
+    }
+}
+
+impl TryFrom<Direction> for ProtobufMoveTabDirection {
+    type Error = &'static str;
+    fn try_from(direction: Direction) -> Result<Self, &'static str> {
+        match direction {
+            Direction::Left => Ok(ProtobufMoveTabDirection::Left),
+            Direction::Right => Ok(ProtobufMoveTabDirection::Right),
+            _ => Err("Wrong direction for ProtobufMoveTabDirection"),
+        }
+    }
+}
+
 impl TryFrom<ProtobufRunCommandAction> for RunCommandAction {
     type Error = &'static str;
     fn try_from(
